/*
	Copyright (C) 2011 Nicklas Nordborg

	This file is part of the Example Code Package for BASE.
	Available at http://baseplugins.thep.lu.se/
	BASE main site: http://base.thep.lu.se/
	
	This is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	The software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.examples.extensions.edit;

import javax.servlet.ServletRequest;

import net.sf.basedb.clients.web.extensions.AbstractJspActionFactory;
import net.sf.basedb.clients.web.extensions.JspContext;
import net.sf.basedb.clients.web.extensions.edit.AbstractOnSaveAction;
import net.sf.basedb.clients.web.extensions.edit.OnSaveAction;
import net.sf.basedb.core.AnyToAny;
import net.sf.basedb.core.BasicItem;
import net.sf.basedb.core.DbControl;
import net.sf.basedb.core.File;
import net.sf.basedb.util.Values;
import net.sf.basedb.util.extensions.InvokationContext;

/**
	Example on-save action factory that saves an any-to-any link
	between a sample/extract and it's history file. This action is the
	companion to the "History file" action that adds an extra
	tab in the "Edit sample/extract" dialogs.
	
	@author nicklas
*/
public class HistoryOnSaveActionFactory 
	extends AbstractJspActionFactory<OnSaveAction>
{

	public HistoryOnSaveActionFactory()
	{}

	/*
		From the ActionFactory interface
		--------------------------------
	*/
	@Override
	public OnSaveAction[] getActions(InvokationContext<? super OnSaveAction> context)
	{
		return new OnSaveAction[] 
		{
			new HistoryOnSaveAction((JspContext)context.getClientContext())			
		};
	}
	// ---------------------------
	
	
	/**
		The action class is doing the actual work. We extend
		AbstractOnSaveAction for future compatibility.
	*/
	static class HistoryOnSaveAction
		extends AbstractOnSaveAction
	{
	
		/**
			Create a new action for the given context.
			The current item can be retreived by {@link JspContext#getCurrentItem()}.
		*/
		HistoryOnSaveAction(JspContext context)
		{
			setJspContext(context);
		}

		/*
			From the OnSaveAction interface
			-------------------------------
		*/
		@Override
		public void onSave()
		{
			JspContext jsp = getJspContext();

			// Get the id and description of the selected file from the http request
			// -1 =  no change (or the user was denied access)
			ServletRequest request = jsp.getPageContext().getRequest();
			int fileId = Values.getInt(request.getParameter("history_file_id"), -1);
			String historyDescription = 
				Values.getStringOrNull(request.getParameter("history_description"));

			
			// Get the current sample
			BasicItem item = (BasicItem)jsp.getCurrentItem();

			DbControl dc = jsp.getDbControl();
			AnyToAny any = null;
			
			// We need to handle three cases:
			// fileId == 0: No file selected
			// fileId > 0: A different file has been selected
			// fileId < 0: The file has not changed
			
			if (fileId == 0)
			{
				// Remove the current any-to-any link
				if (item.isInDatabase()) AnyToAny.unlinkFrom(dc, item, "history-file");
			}
			else if (fileId > 0)
			{
				// Load the file object
				File file = File.getById(dc, fileId);
				
				// Link to the file (automatically create new AnyToAny if needed)
				any = AnyToAny.getNewOrExisting(dc, item, "history-file", file, true);
			}
			else
			{
				// Load the existing link
				any = AnyToAny.getByName(dc, item, "history-file");
			}
			
			if (any != null)
			{
				// Update description
				any.setDescription(historyDescription);
			
				// Save the link if it isn't already in the database
				if (!any.isInDatabase()) dc.saveItem(any);
			}
		}
		
		// -------------------------------------
	}
	
}
